@@ -1,13 +1,14 @@
 """Weaving code."""
 import json
+from multiprocessing import Pool
 from pathlib import Path
 from typing import Dict, Tuple
-from multiprocessing import Pool
 
-import tqdm
-import numpy as np
+import cairo
 from cv2 import cv2
+import numpy as np
 from skimage.measure import profile_line
+import tqdm
 
 from pyanchorknit.utils import config_logger
 
@@ -48,6 +49,7 @@ def weaving(
     n_edges: int = 512,
     maxlines: int = 2000,
     n_jobs: int = 8,
+    intensity_threshold: float = 0.1,
 ):
     logger.info(f"image path: {img_path}")
     logger.info(f"number of edges: {n_edges}")
@@ -60,6 +62,9 @@ def weaving(
     centerx = img.shape[0] // 2
     centery = img.shape[1] // 2
 
+    side = img.shape[0]
+    halfside = side / 2
+
     points = []
 
     for theta in np.linspace(0, 2 * np.pi, n_edges + 1)[:-1]:
@@ -67,7 +72,18 @@ def weaving(
         y = min(int(centery * np.sin(theta) + centery), img.shape[1] - 1)
         points.append(np.array([x, y], dtype=np.int32))
 
-    white_img: np.typing.ArrayLike = np.ones(img.shape[:2], dtype=np.uint8) * WHITE
+    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, *img.shape[:2])
+    ctx = cairo.Context(surface)
+    ctx.set_line_width(0.5)
+
+    ctx.set_source_rgb(1, 1, 1)
+    ctx.arc(halfside, halfside, halfside, 0, 2 * np.pi)
+    ctx.clip()
+
+    ctx.rectangle(0, 0, side, side)
+    ctx.fill()
+
+    ctx.set_source_rgb(0, 0, 0)
 
     traces: Dict[Tuple[int, int], float] = dict()
     thread_length = 0
@@ -75,6 +91,8 @@ def weaving(
     p = 0
     max_intensity = None
 
+    ctx.move_to(*tuple(points[p]))
+
     for _ in tqdm.tqdm(range(maxlines)):
         plines = [(p, j) for j in range(len(points)) if j != p]
 
@@ -89,7 +107,7 @@ def weaving(
 
         max_intensity = max(max_intensity or 0, intensity)  # flip by color
 
-        if len(traces) > 0 and (intensity / max_intensity) < 0.10:
+        if len(traces) > 0 and (intensity / max_intensity) < intensity_threshold:
             logger.debug("Intensity below threshold")
             break
 
@@ -99,8 +117,10 @@ def weaving(
 
         p1, p2 = tuple(points[p]), tuple(points[pprime])
 
-        bw_img = cv2.line(bw_img, p1, p2, BLACK, 1)
-        white_img = cv2.line(white_img, p1, p2, BLACK, 1)
+        LINEWIDTH = 1
+        bw_img = cv2.line(bw_img, p1, p2, BLACK, LINEWIDTH)
+
+        ctx.line_to(*p2)
 
         thread_length += np.linalg.norm(points[p] - points[pprime])
         traces[(p, pprime)] = intensity
@@ -108,10 +128,12 @@ def weaving(
 
     logger.info(f"Thread used: {thread_length:.2f}")
 
-    circular_mask = create_circular_mask(*img.shape[:2]) * 255
-
-    rgba_img = cv2.merge(np.array([white_img, white_img, white_img, circular_mask]))
-    cv2.imwrite(str(img_out.resolve()), rgba_img)
+    ctx.stroke()
+    # circular_mask = create_circular_mask(*img.shape[:2]) * 255
+    # rgba_img = cv2.merge(np.array([white_img, white_img, white_img, circular_mask]))
+    # cv2.imwrite(str(img_out.resolve()), rgba_img)
+    with open(img_out, "wb") as fileobj:
+        surface.write_to_png(fileobj)
 
     points_json = {i: tuple(map(int, point)) for i, point in enumerate(points)}
     traces_json = [((x, y), trace_len) for (x, y), trace_len in traces.items()]
